<?php
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_soap.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_cache.php');

class tx_mageconnect_flexform
{
	protected $_products;
	protected $_soap;
	protected $_helper;
	protected $_cache;
	
	function __construct()
	{
		$this->_helper 	= t3lib_div::makeInstance('tx_mageconnect_helper');
	}
	
	function _connect()
	{
		$soapUrl 	= $this->_helper->getModuleConfig('magento_url');
		$soapUser 	= $this->_helper->getModuleConfig('magento_api_username');
		$soapKey 	= $this->_helper->getModuleConfig('magento_api_key');
		
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	/**
	 * initCache
	 * initialize Caching Instance
	 */
	function initCache()
	{
		$this->_cache = t3lib_div::makeInstance('tx_mageconnect_cache');
	}

	/**
	 * populateProductList
	 * Populates the product option list in flexform
	 * 
	 * @param $config
	 * @return array
	 */
	function populateProductList($config)
	{
		$this->initCache();
		
		// Cache Lifetime
		$lifetime = $this->_helper->getModuleConfig('cache_lifetime');
		
		$cacheId = 'productsList';
		if (FALSE === ($optionList = $this->_cache->get($cacheId))) {
			
			$optionList = array();
		
			if (empty($this->_products)) {
					
				try {
					$this->_connect();
					$this->_products = $this->_soap->getProductsList();
				} catch (Exception $e) {
					die('Could not fetch products list: ' . $e);
				}
				
			} 
			
			foreach ($this->_products as $_product) {
					
				$productId 		= $_product->product_id;
				$productSku		= $_product->sku;
				$productName 	= $_product->name;
				
				$optionStr = '['.$productId.'] ' . $productName;
				
				$optionList[] = array(0 => $optionStr, 1 => $productId);
			}
			$this->_cache->set( $cacheId, $optionList, array(), $lifetime );
		
		} 
		$config['items'] = array_merge($config['items'],$optionList);
		return $config;	
	}
}

