<?php
/**
 * MAGECONNECT
 * Connecting TYPO3 and Magento
 * 
 * Fetch Product Class
 * 
 * This class fetches product details from
 * TYPO3 and Magento
 * 
 * @author Bastian 'Aggy' Zagar
 * @version 0.1
 *
 */

require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_soap.php');

class tx_mageconnect_product extends tx_mageconnect_pi2
{
	/**
	 * Soap Class
	 * @var tx_mageconnect_soap
	 */	
	private $_soap;
	
	private $pObj;
	public  $conf;
	
	function __construct()
	{
		parent::__construct();
	}
	
	function __destruct()
	{
		
	}
	
	/**
	 * init
	 * Initialize the given cObj
	 * 
	 * @param $cObj
	 */
	function init(&$pObj, &$conf)
	{
		$this->pObj = $pObj;
		$this->conf = $conf;
		$this->pi_loadLL();
	}
	
	
	/**
	 * _connect
	 * Connect to webservice
	 */
	function _connect()
	{
		$soapUrl 	= $this->_helper->getModuleConfig('magento_url');
		$soapUser 	= $this->_helper->getModuleConfig('magento_api_username');
		$soapKey 	= $this->_helper->getModuleConfig('magento_api_key');
		
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	/**
	 * getImage
	 * Gets an resized image; 
	 * Looks, what display type has been chosen
	 * 
	 * @param string $image
	 * @return string
	 */
	function getImage($image)
	{
		// Determine image type
		$displayType = $this->_getFlexformConfig('display_type', 'product_setting');
		
		// Check flexform config and fetch settings for image type
		switch ($displayType) {
			
				// Products list
				case 'display_list':
					$imageType = 'listImage';
					break;
				// Products grid
				case 'display_grid':
					$imageType = 'gridImage';
					break;	
				// Single product
				case 'display_single':
					$imageType = 'singleImage';
					break;	
				// Product details
				case 'display_detail':
					$imageType = 'detailImage';
					break;	
				default:
					$imageType = 'listImage';
					break;
				default:
					$imageType = 'listImage';
					break;
		}		
		
		$img = array();
		$img = $this->conf[$imageType.'.'];
		$img['file'] = $this->conf['imagePath'].DIRECTORY_SEPARATOR.$image;
		
		$image = $this->pObj->IMAGE($img);
		return $image;
	}
	
	
	/**
	 * getProductMarkers
	 * Gets all possible product
	 * markers
	 * 
	 * @param array $product Product Data Array
	 * @return array
	 */
	public function getProductMarkers($product)
	{
		if (array_key_exists('attributes', $product)) {
			$attributes = unserialize($product['attributes']);
			
			foreach ($attributes as $_code=>$_val) {
				if (is_array($_val)) {
					unset($attributes[$_code]);
				}
			}
			
			$attributeMarkers = $this->_helper->getMarkers($attributes, 'DETAIL');
			
			unset($product['attributes']);
		}
		$markers = $this->_helper->getMarkers($product);
		
		// Product Image Marker
		if (array_key_exists('image', $product)) {
			$markers['###IMAGE###'] = $this->getImage($product['image']);
		}
		
		if (isset($attributeMarkers)) {
			$markers = t3lib_div::array_merge_recursive_overrule($attributeMarkers, $markers);
		}
		
		// SPECIAL PRODUCTS MARKERS
		$markers['###STOCK###']	= $this->pi_getLL('template.label_in_stock') . ' ' . (int)$product['stock_qty'];
		
		if ($product['stock_qty'] > 0) {
			$markers['###AVAILABILITY###'] = '<span class=\'available\'>'.$this->pi_getLL('template.label_available').'</span>';
		} else {
			$markers['###AVAILABILITY###'] = '<span class=\'not_available\'>'.$this->pi_getLL('template.label_not_available').'</span>';
		}
		
		if ($product['special_price'] != '') {
			$markers['###PRICE###'] = $product['price'];
			$markers['###SPECIAL_PRICE###'] = $product['special_price'];
		} else {
			$markers['###PRICE###'] = '<span class="price"></span>';
			$markers['###SPECIAL_PRICE###'] = $product['price'];
		}
		
		$markers = t3lib_div::array_merge_recursive_overrule($markers, $this->getGlobalMarkers());
		return $markers;
	}
	
	/**
	 * getGlobalMarkers
	 * Gets global markers for every need
	 * 
	 * @return array
	 */
	function getGlobalMarkers()
	{
		$globMarkers = array();
		$globMarkers['###MAGENTO_BASE_URL###'] 				= $this->_helper->getMagentoUrl();
		$globMarkers['###MAGENTO_MEDIA_URL###'] 			= trim($this->_helper->getMagentoUrl(),'/') .'/'.'media';
		$globMarkers['###MAGENTO_PRODUCT_IMAGE_URL###'] 	= trim($this->_helper->getMagentoUrl(),'/') .'/'.'media/catalog/product';
		$globMarkers['###MAGENTO_ADDTOCART_URL###'] 		= trim($this->_helper->getMagentoUrl(),'/') .'/'.'checkout/cart/add/product/';
		
		// Labels
		$globMarkers['###LABEL_ADDTOCART###'] 				= $this->pi_getLL('template.label_add_to_cart');
		$globMarkers['###LABEL_AVAILABILITY###'] 			= $this->pi_getLL('template.label_availability');
		$globMarkers['###LABEL_AVAILABLE###'] 				= $this->pi_getLL('template.label_available');
		$globMarkers['###LABEL_NOT_AVAILABLE###'] 			= $this->pi_getLL('template.label_not_available');
		$globMarkers['###LABEL_IN_STOCK###'] 				= $this->pi_getLL('template.label_in_stock');
		$globMarkers['###LABEL_PRICE###'] 					= $this->pi_getLL('template.label_price');
		$globMarkers['###LABEL_SPECIAL_PRICE###'] 			= $this->pi_getLL('template.label_special_price');
		$globMarkers['###LABEL_ADDITIONAL_INFORMATION###']  = $this->pi_getLL('template.label_additional_information');
		
		return $globMarkers;
	}	
	
	
	/**
	 * getProductsFromDb
	 * Gets product details from database
	 * 
	 * @param array $productIds
	 * @return array
	 */
	function getProductsFromDb($productIds = array())
	{
		$products = array();
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 							// SELECT
															 'tx_mageconnect_products', 	// FROM
															 'product_id IN ('.implode(',',$productIds).') 
															 	AND hidden = \'0\' 
															 	AND deleted = \'0\'',		// WHERE
															 'product_id',					// GROUP BY
															 '',							// ORDER BY
															 ''								// LIMIT
															);
		if ($dbResource) {
			
			while($_product = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource)) {
				$products[] = $_product;
			}
		}
		return $products;
	}
	
	/**
	 * getExistingProducts
	 * Gets all existing products in database
	 * 
	 * @return array
	 */
	function getExistingProducts()
	{
		$products = array();
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', 'tx_mageconnect_products');
		if ($dbResource) {
			
			$rows = array();
			while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource)) {
				$rows[] = $row;
			}
		}
		foreach ($rows as $_row) {
			$products[] = $_row['uid'];
		}
		
		return $products;
	}	
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		return $this->pi_getFFValue($this->pObj->data['pi_flexform'], $field, $type); 
	}	
		
	/**
	 * importImage
	 * Imports an image from a given image path
	 * 
	 * @param string $img
	 */	
	function importImage($img) 
	{
		$mediaUrl = trim($this->_helper->getMagentoUrl(),'/') .'/'.'media/catalog/product';
		$uploadPath = $this->conf['imagePath'] . DIRECTORY_SEPARATOR;
		
		$file = basename($img);
		$imgfile = $uploadPath . $file;
		
		if(!is_file($imgfile)) {
			$img 	= file_get_contents($mediaUrl . $img);
			
			if (!t3lib_div::writeFile($imgfile, $img)) {
				return false;
			}
		}
		return $file;
	}	
		
		
	/**
	 * importProducts
	 * Import given product ids to database for
	 * better performance
	 * 
	 * @param $productIds array
	 * @return bool
	 */	
	function importProducts($productIds = array()) 
	{
		$this->_connect();
		
		$existingIds = $this->getExistingProducts();
		$status = null;
		foreach ($productIds as $_id) {
			
			$productData = $this->_soap->getProductById($_id);

			// Product Image
			$image = '';
			if (array_key_exists('image', $productData)) {
				if ($productData['image'] != 'no_selection') {
					$image = $this->importImage($productData['image']);
				}
			}

			if (t3lib_div::inArray($existingIds, $_id)) {
				$product = array(
					'pid'					=> $this->_getFlexformConfig('productsPid', 'general_setting'),
					'product_id'			=> $productData['product_id'],
					'sku'					=> $productData['sku'],
					'name'					=> $productData['name'],
					'short_description'		=> $productData['short_description'],
					'description'			=> $productData['description'],
					'price'					=> $productData['price'],
					'special_price'			=> $productData['special_price'],
					'stock_qty'				=> $productData['qty'],
					'image'					=> $image,
					'attributes'			=> serialize($productData)
				);
				$result = $this->updateProduct($_id, $product);
				
			} else {
				$product = array(
					'uid'					=> $productData['product_id'],
					'pid'					=> $this->_getFlexformConfig('productsPid', 'general_setting'),
					'product_id'			=> $productData['product_id'],
					'sku'					=> $productData['sku'],
					'name'					=> $productData['name'],
					'short_description'		=> $productData['short_description'],
					'description'			=> $productData['description'],
					'price'					=> $productData['price'],
					'special_price'			=> $productData['special_price'],
					'stock_qty'				=> $productData['qty'],
					'image'					=> $image,
					'attributes'			=> serialize($productData)
				);
				$result = $this->insertProduct($product);
			}
			if ($result !== true) {
				$status = false;
			}
		}
		return ($status === null)?true:false;
	}	

	/**
	 * insertProduct
	 * Inserts an product to database
	 * 
	 * @param array $productData
	 * @return bool
	 */
	public function insertProduct($productData)
	{
		try {
			$insertQuery = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_mageconnect_products', $productData);
		
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}
	
	/**
	 * updateProduct
	 * Updates an product in the database
	 * 
	 * @param int $productId
	 * @param array $productData
	 * @return bool
	 */
	public function updateProduct($productId, $productData)
	{
		try {
			$updateQuery = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_mageconnect_products', 'product_id=\''. $productId . '\' OR uid = \''. $productId . '\'', $productData);
		
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}		
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_product.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_product.php']);
}
