<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2011 Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 */
require_once(PATH_tslib.'class.tslib_pibase.php');

require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_user.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_crypt.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');

/**
 * Plugin 'MageConnect Login Form' for the 'mageconnect' extension.
 *
 * @author	Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
 * @package	TYPO3
 * @subpackage tx_mageconnect
 */
class tx_mageconnect_pi1 extends tslib_pibase {
	
	var $prefixId      = 'tx_mageconnect_pi1';		// Same as class name
	var $scriptRelPath = 'pi1/class.tx_mageconnect_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'mageconnect';	// The extension key.
	//CACHING var $pi_checkCHash = true;
	
	var $templateHtml;
	
	var $_crypt;
	var $_callUser;
	var $_cust;
	var $_helper;
	var $conf;
	
	
	function __construct()
	{
		$this->_helper = t3lib_div::makeInstance('tx_mageconnect_helper'); 
	}
	
	/**
	 * _importUserFromMagento
	 * Imports an user by email from magento
	 * to TYPO3 via SOAP
	 * 
	 * @param string $email
	 * @return bool
	 */
	function _importUserFromMagento($email)
	{
		if (!$email) {
			return false;
		}
		
		$this->_callUser = t3lib_div::makeInstance('tx_mageconnect_user'); 
		$customerData =	$this->_callUser->getMagentoUserByEmail($email);
		
		if (property_exists($customerData, 'attributes')) {
		
			// Write the userdata into the locked field
			$customAttributesStr = '';
			foreach ($customerData->attributes as $_attrCode=>$_attrVal) {
				
				// Generate String for Backend Insert
				$customAttributesStr .= $_attrCode .'::'. $_attrVal .'*'; 
			}
			
			$userArray = array();
			
			// If User exists in TYPO3 -> UPDATE
			if ($this->_callUser->checkTYPO3UserExists($email) == true)
			{
				$userArray = array(
					'pid' 			=> $this->_getFlexformConfig('userPid','general_setting'),
					'tstamp' 		=> time(),
					'username' 		=> $customerData->attributes->email,
					'password' 		=> $customerData->attributes->password_hash,
					'name'			=> trim($customerData->attributes->prefix	. ' ' .
									   $customerData->attributes->firstname 	. ' ' . 
									   $customerData->attributes->middlename 	. ' ' .
									   $customerData->attributes->lastname),
					'first_name'	=> $customerData->attributes->firstname,
					'middle_name'	=> $customerData->attributes->middlename,
					'last_name'		=> $customerData->attributes->lastname,

					'email'			=> $customerData->attributes->email,
					'crdate'		=> time(),
					'tx_mageconnect_magento_customer_attr' => $customAttributesStr
				);
				
				if ( property_exists($customerData, 'billing') ) {
				
					$userArray = array_merge($userArray, array(
						'address'		=> $customerData->billing->street,
						'telephone'		=> $customerData->billing->telephone,
						'fax'			=> $customerData->billing->fax,
						'zip'			=> $customerData->billing->postcode,
						'city'			=> $customerData->billing->city,
						'country'		=> $customerData->billing->country_id,
						'company'		=> $customerData->billing->company,
						)
					);
				}
				
				if ( $this->_callUser->updateTYPO3User($email, $userArray) ) {
					$this->_cust = $customerData;
				}
				
			// If User doesn't exist in TYPO3 -> INSERT	
			} else {
				
				$userArray = array(
					'pid' 			=> $this->_getFlexformConfig('userPid','general_setting'),
					'tstamp' 		=> time(),
					'username' 		=> $customerData->attributes->email,
					'password' 		=> $customerData->attributes->password_hash,
					'usergroup' 	=> $this->_getFlexformConfig('userGroupPid','general_setting'),
					'name'			=> trim($customerData->attributes->prefix	. ' ' .
									   $customerData->attributes->firstname 	. ' ' . 
									   $customerData->attributes->middlename 	. ' ' .
									   $customerData->attributes->lastname),
					'first_name'	=> $customerData->attributes->firstname,
					'middle_name'	=> $customerData->attributes->middlename,
					'last_name'		=> $customerData->attributes->lastname,
					'email'			=> $customerData->attributes->email,
					'crdate'		=> time(),
					'tx_mageconnect_magento_customer_attr' => $customAttributesStr
				);
				
				if ( property_exists($customerData, 'billing') ) {
				
					$userArray = array_merge($userArray, array(
						'address'		=> $customerData->billing->street,
						'telephone'		=> $customerData->billing->telephone,
						'fax'			=> $customerData->billing->fax,
						'zip'			=> $customerData->billing->postcode,
						'city'			=> $customerData->billing->city,
						'country'		=> $customerData->billing->country_id,
						'company'		=> $customerData->billing->company,
						)
					);
				}
				
				// Call INSERT
				if ( $this->_callUser->insertTYPO3User($userArray) ) {
					$this->_cust = $customerData;
				}
			}
			return true;
		}
		return false;
	}

	/**
	 * _loginUser
	 * Login a user with given credentials
	 * 
	 * @param string $username
	 * @param string $password
	 * @return bool
	 */
	function _loginUser($username, $password)
	{
		$loginData=array(
			'uname' => $username, //usernmae
			'uident'=> $password, //password
			'status' =>'login'
		);
		
		$GLOBALS['TSFE']->fe_user->checkPid 	= 0;
		$info 									= $GLOBALS['TSFE']->fe_user->getAuthInfoArray();
		$user 									= $GLOBALS['TSFE']->fe_user->fetchUserRecord($info['db_user'] ,$loginData['uname']);
		$ok 									= $GLOBALS['TSFE']->fe_user->compareUident($user, $loginData);

		if($ok) {
			$GLOBALS['TSFE']->fe_user->createUserSession($user);
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * _decryptStr
	 * Decrypts an given string
	 * 
	 * @param string $string
	 * @return string
	 */
	function _decryptStr($string) 
	{
		if (empty($this->_crypt)) {
			$this->_crypt = t3lib_div::makeInstance('tx_mageconnect_crypt', $this->_helper->getModuleConfig('encrypt_decrypt_key') );
		}
		return $this->_crypt->getDecrypted( $string );
	}
	
	/**
	 * _getLoginForm
	 * Displays the login form from the
	 * template
	 * 
	 * @return string
	 */
	function _getLoginForm()
	{
		// Load Template File
		$this->templateHtml = $this->cObj->fileResource( $this->_getConfig('templateLogin') );
		
		// Get frontend login marker
		$frontendLogin = $this->cObj->getSubpart($this->templateHtml, '###FRONTEND_LOGIN###');
		$markers['###TYPO3CONNECT_URL###'] = $this->_helper->getTYPO3connectUrl();
		$markers['###REDIRECT_URL###'] = $this->_getRedirectUrl();
		$markers['###FORGOT_PASSWORD_URL###'] = $this->_helper->getMagentoUrl() . 'customer/account/forgotpassword/';
		
		// Labels
		$markers['###LABEL_EMAIL###'] = $this->pi_getLL('template.label_email');
		$markers['###LABEL_PASSWORD###'] = $this->pi_getLL('template.label_password');
		$markers['###LABEL_LOGIN###'] = $this->pi_getLL('template.label_login');
		$markers['###LABEL_FORGOT_PASSWORD###'] = $this->pi_getLL('template.label_forgot_password');
		
		return $this->cObj->substituteMarkerArrayCached($frontendLogin, $markers);
	}
	

	
	/**
	 * _getLoginSuccessMessage
	 * Displays the login success message and fills
	 * out dynamicly generated markers from
	 * magento customer attributes
	 * 
	 * @return string
	 */
	function _getLoginSuccessMessage()
	{
		// Load Template File
		$this->templateHtml = $this->cObj->fileResource( $this->_getConfig('templateLogin') );
		
		// Get frontend login marker
		$successTemplate = $this->cObj->getSubpart($this->templateHtml, '###SUCCESS_TEMPLATE###');
		
		if ($this->_cust) {
			
			$markers = array();
			
			// ACCOUNT MARKERS
			$accMarkers = $this->_helper->getMarkers( $this->_cust->attributes );
			$markers = array_merge( $markers, $accMarkers );
			
			// BILLING MARKERS
			$billMarkers = $this->_helper->getMarkers( $this->_cust->billing, 'BILLING' );
			$markers = array_merge( $markers, $billMarkers );
			
			// SHIPPING MARKERS
			$shipMarkers = $this->_helper->getMarkers( $this->_cust->shipping, 'SHIPPING' );
			$markers = array_merge( $markers, $shipMarkers);
			
			return $this->cObj->substituteMarkerArrayCached($successTemplate, $markers);
		}
		return '';
	}
	
	/**
	 * _getLoginForm
	 * Displays the login form from the
	 * template
	 * 
	 * @param string $errormsg
	 * @return string
	 */
	function _getErrorTemplate($errormsg)
	{
		// Load Template File
		$this->templateHtml = $this->cObj->fileResource( $this->_getConfig('templateLogin') );
		
		// Get frontend login marker
		$frontendLogin = $this->cObj->getSubpart($this->templateHtml, '###ERROR_TEMPLATE###');
		$markers['###BASE_URL###'] = $this->_helper->getScriptUrl();
		$markers['###ERROR_MESSAGE###'] = $errormsg;
		
		// Labels
		$markers['###LABEL_TRY_AGAIN###'] = $this->pi_getLL('template.label_try_again');
		
		return $this->cObj->substituteMarkerArrayCached($frontendLogin, $markers);
	}
	
	/**
	 * _getConfig
	 * Return config data from a given
	 * key
	 * 
	 * @param string $key
	 * @return string
	 */
	function _getConfig($key)
	{
		return $this->conf[$key];
	}
	
	/**
	 * _getRedirectUrl
	 * Determine the redirect url from flexform
	 * configuration
	 * 
	 * @return string
	 */
	function _getRedirectUrl()
	{		
		if($this->_getFlexformConfig('redirect_custom_url','redirect_setting'))
		{
			return $this->_getFlexformConfig('redirect_custom_url','redirect_setting');
		} else {
			
			$redirectSetting = $this->_getFlexformConfig('redirect_destination','redirect_setting');
			
			switch ($redirectSetting) 
			{
				case 'redirect_typo3':
					return $this->_helper->getScriptUrl();
				
				case 'redirect_magento':
					return $this->_helper->getModuleConfig('magento_url');
				
				case 'redirect_magento_dashboard':
					return $this->_helper->getModuleConfig('magento_url') . 'customer/account/';
					
				default:
					return $this->_helper->getScriptUrl();
			}
		}
	}
	
	/**
	 * _redirect
	 * Redirects to another url
	 * 
	 * @param string $url
	 */
	function _redirect($url)
	{
		header('location: ' . $url);
	}
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		return $this->pi_getFFValue($this->cObj->data['pi_flexform'], $field, $type); 
	}
	
	/**
	 * _getVars
	 * Gets specific post vars and returns an 
	 * array
	 * 
	 * @return array
	 */
	function _getVars()
	{
		$vars = array(	'username' => t3lib_div::_GP('username'),
						'password' => t3lib_div::_GP('password')
		);
		return $vars;
	}
	
	/**
	 * _getHash
	 * Gets the password hash from
	 * a given username
	 * 
	 * @param string $username
	 * @return string
	 */
	function _getHash($username)
	{
		if (empty($this->_callUser)) {
			$this->_callUser = new FetchUser(); 
		}
		return $this->_callUser->getPasswordHash($username);
	}
		
	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content, $conf) 
	{
		$this->pi_initPIflexForm();    // Initialize the FlexForms array 
		
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		// DISABLE CACHING
		$this->pi_USER_INT_obj = 1;
		
		
		// Standard display
		if ($this->_helper->_getParam('data') != false) {
			
			// Create data variables
			$divider		= ':-:-:';
			$data 			= explode($divider, $this->_helper->_getParam('data') );
			
			$redirectUrl	= $this->_decryptStr( $data[0] );
			$timestamp 		= $this->_decryptStr( $data[1] );
			$username 	 	= $this->_decryptStr( $data[2] );
			$password	 	= $this->_decryptStr( $data[3] );
			$postedVars	 	= $this->_getVars();
			
			// Active Login duration for maximum 2 Minutes
			if ( 	(date("d.m.Y H") == date("d.m.Y H", $timestamp) ) 	&& 
					( time() < $timestamp+2*60 )						&&
					$postedVars['username'] == $username				&&
					$postedVars['password'] == $password				
			) {
				
				if ($this->_importUserFromMagento($username) == true) {
					$hash = $this->_getHash($username);

					// Is password correct to the database?
					if ( ( $this->_callUser->magentoPasswordCompare($password, $hash) == true)
					|| ( $this->_callUser->magentoPasswordCompare($postedVars['password'], $hash) == true )
					)
					{
						// If user is logged in, redirect to custom setting
						if ($this->_loginUser($username, $hash) == true) {
							
							$content = $this->_getLoginSuccessMessage();
							
							$show_customer = $this->_getFlexformConfig('show_customer','redirect_setting');
							
							if ($show_customer != true) {
								
								if ($redirectUrl != $this->_getRedirectUrl()) {
									header('location:' . $redirectUrl);
									//t3lib_utility_Http::redirect($redirectUrl, HTTP_STATUS_200);
								} else {
									header('location:' . $this->_getRedirectUrl());
									//t3lib_utility_Http::redirect($this->_getRedirectUrl(), HTTP_STATUS_200);
								}
							}
							
						} else {
							// Some error!?
							$content = $this->_getErrorTemplate( $this->pi_getLL('template.label_could_not_login') );
						}
					}
				}
			} else {
				// Tried to execute link manually
				// or inactive!
				$content = $this->_getErrorTemplate( $this->pi_getLL('template.label_session_inactive') );
			}
			
		} else if ($this->_helper->_getParam('error') != false) {
			$errorMessage = $this->_decryptStr( $this->_helper->_getParam('error') );
			$content = $this->_getErrorTemplate( $errorMessage );
					
		} else {
			
			if (!$GLOBALS['TSFE']->fe_user->user) {
				$content = $this->_getLoginForm();
			} else {
				$content = '';
			}
			
		}
		return $this->pi_wrapInBaseClass($content);
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi1/class.tx_mageconnect_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi1/class.tx_mageconnect_pi1.php']);
}


?>